

/** This is an abstract class for the MaxFlowAlgorithm. It provides a base for the specific implementations
 * of the algorithms (Edmonds-Karp , Relabel-to-front) to build on.
 */
public abstract class MaxFlowAlgorithm  {
    
    protected boolean done ;
    protected Graph g ;
    
    /** The constructor
     * @param g the graph to be used
     */
    public MaxFlowAlgorithm(Graph g) {
        this.g= g ;
    }
    
    /** Runs the algorithm on the graph. The algorithm can either be run step wise
     * , or until it completes. The definition of a step is left to the implementation.
     * @param cont if true, the algorithm runs to completion. if false, it will run step -wise.
     * @return a result object
     * @throws Exception if an error occurs within the algorithm
     */
    public abstract Result run(boolean cont) throws Exception ;
    
    
    /** resets the graph, flows through all edges are reset to 0.
     */
    public void reset() {
        done = false ;
        
        for(int i=0 ; i< g.getNVertices() ; i++) {
            AdjacencyList al = g.getAdjList(i) ;
            
            for(int j =0 ; j < al.length() ; j++ ) {
                al.getEdge(j).setFlow(0) ;
            }
            
        }
    }
    
    /** Calculate the current flow through the graph.
     * @return the current flow through the graph
     */
    public int calcFlow() {
        int flow = 0 ;
        // flow is just that going out of the source
        AdjacencyList l = g.getAdjList(g.getSource()) ;
        for (int i = 0; i < l.length() ; i++) {
            flow += l.getEdge(i).getFlow() ;
        }
        return flow ;
    }
    
    protected void printDetails(Result r) {
        for (int i=0; i < g.getNVertices() ; i++) {
            Vertex v = g.getVertex(i) ;
            if( v.isSink() && v != g.getSink() ) {
                AdjacencyList al = g.getAdjList(i) ;
                int sum =0;
                for(int j=0 ; j<al.length() ; j++ ) {
                    int flow = al.getEdge(j).getFlow() ;
                    if(flow > 0)
                        sum += al.getEdge(j).getFlow() ;
                    
                }
                System.out.println("Flow into vertex " + v.getLabel() + " is " + sum) ;
            }
        }
        
        if(r != null) {
            System.out.println("Max Flow: " + r.getFlow() ) ;
        }
        
    }
}
