
import java.util.Vector ;


/** This class represents an Augmentation path as used by the Edmonds Karp algorithm
 *
 */
public class Path {

    protected Graph g ;
    protected Vector edges ;
    protected int bottleneck = -1 ;

    /** Create a new path
     * @param g the graph that this path exists in
     */        
    public Path(Graph g) {
	edges = new Vector() ;
	this.g = g;
    }

    /** add an edge to this path
     * @param e the edge to add
     */        
    public void addEdge(Edge e) {
	edges.add(e) ;
    }

    /** get the number of edges in this path
     * @return the number of edges
     */        
    public int length () {
	return edges.size() ;
    }

    /** get the i'th edge of the path
     * @param i the index of the path to get
     * @return and Edge object
     */        
    public Edge getEdge(int i) {
	return (Edge) edges.get(i) ;
    }

    /** returns the bottle neck (ie min remaining capacity) of this path
     * @return the bottleneck
     */        
    public int getBottleneck() {
    	return bottleneck ;
    }

    /** returns a string representing this path
     * @return a string
     */        
    public String toString() {
	String str = "" ;
	for( int i = 0; i < edges.size()  ; i++) {
            str += ( (Edge) edges.get(i) ).getCap() ;
            str += "," ;
	}
	return str;
    }

    /** Takes a Graph that has had a BFS performed on it,
     * and returns the path from the source to the sink, or null if no path exists
     * @param g the graph
     * @return A Path object, or null if no path exists
     */        
    public static Path fromGraph(Graph g) {
        if (g.getSink().getParent() == null ) {
            return null ;
        }
	
        // ok now we know a path exists
        // follow it back to the source
        Vertex tempV = g.getSink() ;
        Path p = new Path(g) ;
		
        //  goto sink node
        //  step back, adding edges to paths
        //  find bottleneck

        while (tempV != g.getSource()) {
            Edge e = g.getAdjList(tempV.getParent()).getEdgeToVertex(tempV) ;
            p.addEdge(e) ;
            if( p.bottleneck == -1 || ((p.bottleneck > e.getRes() ) && (e.getRes() != -1) ) ) 
                p.bottleneck = e.getRes() ;
            tempV = tempV.getParent() ;
        }
        return p ;
    }
}
