/**
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

import java.util.Vector ;
import java.util.Hashtable ;
import java.util.ArrayList ;

/** This class implements the Relabel to front algorithm for
 * finding the maximium flow in a graph
 */
public class Relabel extends MaxFlowAlgorithm  {

    // this hashtable allows us to lookup the pointer to a vertex in 
    // the adjcentcy list, given a vertex
    // ie. it performs the current[u] operation
    private Hashtable current  ;
    
    // if the pre flow has been initialised
    private boolean running = false ;
    
    // pointer to a vertex under consideration in the arraylist
    private int uIndex = 0 ;
    
    // stores a list of all vertices, minus the source and sink
    // ie. V[G] - {s,t}
    private ArrayList aL = null ;
    
    
    /** Used for the command line program.
     * @param args the commaind line arguments
     */    
    public static void main(String args[]) throws Exception {

	if(args.length == 0) {
		System.out.println("Usage: java Relabel FILE") ;
		return ;
	}
	
        // parse the file, then normalise
	Graph g = Parser.parse(args[0]) ;
	g.normalise() ;

        // create a new object, then call run 
        Relabel r = new Relabel(g) ;
        
	Result res = r.run(true) ;
        
        r.printDetails(res);
    }

    
    /** Create a new object
     * @param g The graph to operate on
     */    
    public Relabel(Graph g) {
	super(g) ;
	current  = new Hashtable() ;
    }
    
    /** Reset the graph
     */    
    public void reset() {
        super.reset() ;
        running = false ;
    }
    
    private void init() {
        Vertex source = g.getSource() ;
	Vertex sink = g.getSink() ;
	
        // init preflow 
        initPreFlow() ;
        aL = new ArrayList() ;

        // add all vertices to arraylist, except the source and sink
        for(int i=0 ; i< g.getNVertices() ; i++) {
            Vertex v = g.getVertex(i) ;
            if( v != source && v != sink) {
                aL.add(v) ;
            }
            // set pointer for this vertex to the start of its adjList
            current.put(v, new Integer(0) ) ;
        }

        // set index to 0, ie. first vertex in arrayList
        uIndex = 0 ;
    }


    /**
     * Discharges the vertex u
     * @param u the vertex to discharge.
     */
    private void discharge(Vertex u) {
	AdjacencyList al = g.getAdjList(u) ;

        int i = ((Integer) current.get(u)).intValue()  ;

	while(u.getExcess() > 0 || u.getExcess() == -1) {
            Vertex v = al.getVertex(i) ;
          
            if(v == null) {
		relabel(u) ;
		i = 0 ;
            } else if ((al.getEdge(i).getRes() == -1 || al.getEdge(i).getRes() > 0) && u.getHeight() == (v.getHeight() + 1)  ) {
		push(u,v) ;
            } else {
		i++ ;
            }
	}
        current.put(u, new Integer(i)) ;

    }

    /*** 
     * pushes some flow from u to v
     */
    private void push(Vertex u, Vertex v) {       
	Edge e = g.getAdjList(u).getEdgeToVertex(v) ;
        int flow ;
                
        if(e.getRes() == -1 ) {
            flow = u.getExcess() ;
        } else if(u.getExcess() == -1) {
            flow = e.getRes() ;
        } else {
            flow = Math.min(u.getExcess() , e.getRes() ) ;
        }
 	Edge eC = e.getComplement() ;
        e.setFlow(e.getFlow() + flow) ;
        eC.setFlow( - e.getFlow() ) ;
        u.setExcess( u.getExcess() - flow) ;
        v.setExcess( v.getExcess() + flow) ;
    }
    
    /**
     * increases the height of u, so that it can be discharged
     */
    private void relabel(Vertex u) {
        AdjacencyList al = g.getAdjList(u) ;
	Vertex min = null;

	for(int j =0 ; j < al.length() ; j++ ) {
            Edge e = al.getEdge(j) ;
            // watch out for -1 here !!!
            if(al.getEdge(j).getRes() > 0 || al.getEdge(j).getRes() == -1) {
		if(min == null) {
                    min = al.getVertex(j) ;
		} else {
                    Vertex v = al.getVertex(j) ;
                    if (v.getHeight() < min.getHeight() ) {
			min = v ;
                    }
                }
            }
	}

	if(min == null) 
            System.out.println("ITS REALLY BAD NOW!!, CAN't FIND AN EDGE") ;
       
	u.setHeight(1 + min.getHeight() ) ;
    }

    /**
     * initialises the pre flow
     */
    private void initPreFlow() {
	Vertex source = g.getSource() ;

	for (int i=0; i<g.getNVertices(); i++) {
            g.getVertex(i).setHeight(0);
            g.getVertex(i).setExcess(0);
	}
		
	for (int i=0; i < g.getNVertices(); i++) {
            Vertex src = g.getVertex(i);
            AdjacencyList l = g.getAdjList(i) ;
			
            for (int j = 0; j < l.length() ; j++ ) {
            	Edge e1 = l.getEdge(j) ;
		e1.setFlow(0) ;
		e1.getComplement().setFlow(0);
            }
	}

	source.setHeight(g.getNVertices());
	
        AdjacencyList adj = g.getAdjList(g.getSource());
          
	for(int k=0;k< adj.length(); k++){
          
            Edge e1 = adj.getEdge(k);
            int cap = e1.getCap() ;
            e1.setFlow(cap);
            e1.getComplement().setFlow(- cap);
		
            adj.getVertex(k).setExcess(cap);
            source.setExcess( source.getExcess() - cap ) ;
	}	

    }

        
    /** Runs the algorithm on the graph. The algorithm can either be run step wise
     * , or until it completes. In this implemenation, one step is the discharging of one vertex
     * @param cont if true, the algorithm runs to completion. if false, it will run step -wise.
     * @return a result object
     * @throws Exception if an error occurs within the algorithm
     */    
    public Result run(boolean cont) throws Exception {
        
	boolean done = true ;
        
        if( !running) {
            init() ;
            running = true ; 
        }
              
        Vertex u = null ;
        
        if(uIndex < aL.size() ) {
            u = (Vertex) aL.get(uIndex) ;
        }
        // store the vertex we consider this iteration
        // used for colouring the GUI
        Vertex old = null ;
        
	while( u != null) {
            // store a reference to this vertex ;
            old = u ;
            
            int old_height = u.getHeight() ;
            discharge(u) ;
            
            if (u.getHeight() > old_height ) {    
		aL.remove(uIndex) ;
		aL.add(0, u) ;
		uIndex = 0 ;
            }
            uIndex++ ;

            
            if(uIndex == aL.size() )
		u = null ;
            else
		u = (Vertex) aL.get(uIndex) ;
            
            if( !cont ) {
                done = false ;
                break ;
            }
        }
        
        Result r = new Result ( calcFlow(), done) ;
        r.setVertex(old) ;
        return r ;
    }


        
}// end class
